@@ -84,8 +84,11 @@ f_substitute <- function (f, env) {
                 assign(n, o[[2]], envir = env)
             }
 
-            # Only copy things the formulae mentions
-            vars_to_copy <- all.names(rlang::f_rhs(o), unique = TRUE)
+            vars_to_copy <- c(
+                # Copy anything the formula explicitly mentions
+                all.names(rlang::f_rhs(o), unique = TRUE),
+                # Anything starting with "001:" (e.g.) is assumed to be an ancillary step, which should be copied regardless
+                grep("^(?:\\d|-)\\d{2}:", names(environment(o)), value = TRUE, perl = TRUE) )
             environment_merge(combined_env, rlang::f_env(o), var_names = vars_to_copy)
         }
     }
@@ -154,7 +157,9 @@ f_concatenate <- function (list_of_f, parent = NULL, wrap_call = NULL) {
 
     orig_e <- e <- parent
     for (f in list_of_f) {
-        if (is.null(orig_e)) {
+        if (is.null(environment(f))) {
+            # f is a call, so has no environment to add
+        } else if (is.null(orig_e)) {
             # At top, keeping previous environment, no need to change env.
             orig_e <- e <- environment(f)
         } else if (identical(environment(f), orig_e)) {
@@ -169,8 +174,10 @@ f_concatenate <- function (list_of_f, parent = NULL, wrap_call = NULL) {
         }
     }
 
-    # Combine all functions into one expression
-    out_call <- as.call(c(list(as.symbol("{")), lapply(unname(list_of_f), rlang::f_rhs)))
+    # Convert all formulas into code, combine into one expression
+    list_of_c <- lapply(list_of_f, function (f) if (rlang::is_formula(f)) rlang::f_rhs(f) else f)
+    out_call <- as.call(c( list(as.symbol("{")), unname(list_of_c) ))
+
     if (!is.null(wrap_call)) {
         # Wrap inner call with outer
         out_call <- as.call(c(
@@ -213,6 +220,21 @@ f_chain_conditional <- function (fs, default_f = NaN, ...) {
     f_substitute(out, fs)
 }
 
+# Combine list of formulas with an operator, e.g. f_chain_op(list(~a, ~b, ~c), "+") ---> ~a + b + c
+f_chain_op <- function (fs, op) {
+    # Assign names to list we can use as symbols
+    names(fs) <- paste0("f", seq_along(fs))
+
+    # Build code to do sum, based list names
+    sum_c <- NULL
+    for (i in seq_along(fs)) {
+        sym <- as.symbol(names(fs)[[i]])
+        sum_c <- if (i == 1) sym else call(op, sum_c, sym)
+    }
+
+    return(f_substitute(sum_c, fs))
+}
+
 # Perform optimizations on code within formulae, mostly for readability
 f_optimize <- function (f) {
     # Simplify Basic arithmetic
@@ -260,6 +282,10 @@ f_optimize <- function (f) {
             x[[2]] <- f_optimize(x[[2]])
             x[[3]] <- f_optimize(x[[3]])
             if (length(x) > 3) {
+                if (is.call(x[[3]]) && identical(x[[3]][[1]], as.symbol("if"))) {
+                    # Have to brace an inner if, otherwise it'll steal our else
+                    x[[3]] <- call("{", x[[3]])  # }
+                }
                 x[[4]] <- f_optimize(x[[4]])
 
                 # If else condition is empty, remove it
@@ -302,6 +328,7 @@ f_optimize <- function (f) {
         },
         "<-" = function (x) {
             if (!is.call(x)) return(x)
+            if (length(x) < 3) return(x) # var <- (missing)
             if (is.call(x[[3]]) && x[[3]][[1]] == "(") {  # )
                 # No point wrapping a definition in braces
                 x[[3]] <- x[[3]][[2]]
@@ -425,13 +452,118 @@ f_eval <- function (f, env_extras = list(), env_parent = g3_env) {
 }
 
 # Find all vars, minus vars that are defined within (e.g. iterators)
-all_undefined_vars <- function (code) {
+all_undefined_vars <- function (code, recursive = FALSE, filter_fn = NULL) {
     g3_with_extract_term_syms <- function (x) {
         lapply(g3_with_extract_terms(x), function (c) as.character(c[[2]]))
     }
 
-    setdiff(all.vars(code), c(
+    out <- setdiff(all.vars(code), c(
         lapply(f_find(code, as.symbol("for")), function (x) { as.character(x[[2]]) }),
         do.call(c, lapply(f_find(code, as.symbol("g3_with")), g3_with_extract_term_syms)),
         NULL))
+    if (recursive) {
+        expanded <- lapply(out, function (n) {
+            # Get undefined vars from anything defined locally
+            defn <- environment(code)[[n]]
+            if (!is.null(filter_fn)) {
+                # Apply filter_fn, which will e.g. resolve stock_ss() and do renames
+                defn <- filter_fn(defn)
+            }
+            if (is.call(defn)) all_undefined_vars(defn, recursive = TRUE, filter_fn = filter_fn) else c()
+        })
+        out <- c(out, unlist(expanded))
+    }
+    return(out)
+}
+
+# Add any formula definitions from f into f, if they include depend_vars
+add_dependent_formula <- function (f, depend_vars, filter_fn = NULL) {
+    extra_defns <- list()
+    wrap_defns <- list()
+
+    # Repeatedly look for definitions we should be adding (so we add sub-definitions)
+    while(TRUE) {
+        # Find any missing definitions in either f or things we're about to define
+        undefined_vars <- c(
+            do.call(c, lapply(extra_defns, all_undefined_vars)),
+            do.call(c, lapply(wrap_defns, all_undefined_vars)),
+            all_undefined_vars(f) )
+        # Ignore things that are already defined
+        undefined_vars <- setdiff(undefined_vars, names(extra_defns))
+        undefined_vars <- setdiff(undefined_vars, names(wrap_defns))
+        added_defn <- FALSE
+        for (var_name in undefined_vars) {
+            for (sub_f in c(list(f), extra_defns, wrap_defns)) {
+                defn <- environment(sub_f)[[var_name]]
+                if (!is.null(defn)) break
+            }
+            if (!is.call(defn)) next
+            if (!is.null(filter_fn)) {
+                # Apply filter_fn, which will e.g. resolve stock_ss() and do renames
+                defn <- filter_fn(defn)
+                assign(var_name, defn, envir = environment(sub_f))
+            }
+            if (TRUE &&
+                    # We have some depend_vars to check
+                    !isTRUE(depend_vars) &&
+                    # Formula doesn't mention any of the depend vars or now-added definitions
+                    length(intersect(all_undefined_vars(defn, recursive = TRUE, filter_fn = filter_fn), c(depend_vars, names(extra_defns)))) == 0 &&
+                    TRUE ) {
+                next
+            }
+
+            if ( !is.null(attr(defn, 'g3_global_init_val')) ) {
+                if (!identical(rlang::f_rhs(defn), as.symbol("noop"))) {
+                    # Update variable instead of adding to extra_defns
+                    wrap_defns[[var_name]] <- defn
+                }
+                # NB: adf_marker will get removed later with collapse_g3_with()
+                extra_defns[[var_name]] <- quote( adf_marker )
+            } else {
+                extra_defns[[var_name]] <- defn
+            }
+            added_defn <- TRUE
+        }
+        if (!added_defn) break
+    }
+
+    if (length(extra_defns) > 0) {
+        ordering <- vapply(extra_defns,
+            # Find earliest var in list that current item depends on
+            function (x) suppressWarnings(min(match(all.vars(x), names(extra_defns)), na.rm = TRUE)),
+            numeric(1) )
+        # Use this as an ordering, so vars without dependencies go first
+        extra_defns <- extra_defns[names(sort(ordering, decreasing = TRUE))]
+
+        # Make up call with all vars to define
+        g3_with_call <- as.call(c(
+            quote(g3_with),
+            # List of defn := defn_adf
+            lapply(names(extra_defns), function (n) call(":=", as.symbol(n), as.symbol(paste0(n, "_adf")))),
+            quote(f) ))
+        # Replace the RHS, not the LHS
+        names(extra_defns) <- paste0(names(extra_defns), "_adf")
+        # Wrap f in a g3_call()
+        f <- f_substitute(g3_with_call, c(extra_defns, list(f = f)))
+    }
+
+    for (var_name in names(wrap_defns)) {
+        ordering <- vapply(wrap_defns,
+            # Find earliest var in list that current item depends on
+            function (x) suppressWarnings(min(match(all.vars(x), names(wrap_defns)), na.rm = TRUE)),
+            numeric(1) )
+        # Use this as an ordering, so vars without dependencies go first
+        wrap_defns <- wrap_defns[names(sort(ordering, decreasing = TRUE))]
+
+        # Put any g3_global_formula iterations on the outside
+        # NB: This is somewhat abritary, but then the precise positioning of a g3_global_formula is a bit ropey anyway.
+        f <- f_substitute(quote(
+            { var <- defn; f }
+        ), list(
+            var = as.symbol(var_name),
+            defn = wrap_defns[[var_name]],
+            f = f ))
+    }
+
+    return(f)
 }
