@@ -2,14 +2,19 @@ open_curly_bracket <- intToUtf8(123) # Don't mention the bracket, so code editor
 
 # Compile actions together into a single R function,
 # The attached environment contains model_data, i.e. fixed values refered to within function
-g3_to_r <- function(actions, trace = FALSE, strict = FALSE) {
+g3_to_r <- function(
+        actions,
+        work_dir = getOption('gadget3.r.work_dir', default = tempdir()),
+        trace = FALSE,
+        strict = FALSE,
+        cmp_options = list(optimize = 3) ) {
     collated_actions <- g3_collate(actions)
     all_actions <- f_concatenate(c(
-        g3_formula(quote(cur_time <- cur_time + 1L), cur_time = -1L),
         collated_actions,
         NULL), parent = g3_env, wrap_call = call("while", TRUE))
-    # NB: Needs to be globalenv() to evaluate core R
-    model_env <- new.env(parent = globalenv())
+    # NB: R envs look like globalenv() <- package:attached <- package:attached <- .. <- baseenv() <- emptyenv()
+    #     Attach to one up from globalenv()
+    model_env <- new.env(parent = parent.env(globalenv()))
     scope <- list()
 
     # R always reports
@@ -32,14 +37,16 @@ g3_to_r <- function(actions, trace = FALSE, strict = FALSE) {
         # Replace any in-line g3 calls that may have been in formulae
         repl_fn <- function(x) {
             # NB: eval() because -1 won't be a symbol
-            find_arg <- function (arg_name, def, do_eval = TRUE) {
-                if (!(arg_name %in% names(x))) return(def)
-                if (do_eval) return(eval(x[[arg_name]], envir = env))
-                return(x[[arg_name]])
+            find_arg <- function (arg_name, def, do_eval = TRUE, sub_param_idx = NULL) {
+                out <- if (arg_name %in% names(x)) x[[arg_name]] else def
+                if (do_eval) out <- eval(out, envir = env)
+                # For g3_param_table(..., value = 1:4), split up vector into individual positions
+                if (!is.null(sub_param_idx) && length(out) > 1) out <- out[[sub_param_idx]]
+                return(out)
             }
-            df_template <- function (name, dims = c(1)) {
+            df_template <- function (name, dims = c(1), sub_param_idx = NULL) {
                 # Extract named args from g3_param() call
-                value <- eval(find_arg('value', 0), envir = env)
+                value <- find_arg('value', 0, sub_param_idx = sub_param_idx)
 
                 structure(list(value), names = name)
             }
@@ -52,31 +59,45 @@ g3_to_r <- function(actions, trace = FALSE, strict = FALSE) {
                 ifmissing <- call_replace(ifmissing,
                     g3_param_table = repl_fn,
                     g3_param = repl_fn)
+                pt_name <- paste0("pt.", x[[2]])
 
                 # NB: We eval, so they can be defined in-formulae
                 df <- eval(x[[3]], envir = env)
 
-                # Add stopifnot for each row in table
+                # Buld lookup list, ensure each row is present at runtime
+                init_list = list()
                 for (i in seq_len(nrow(df))) {
-                    sub_param_name <- gen_param_tbl_name(as.character(x[[2]]), df[i,])
+                    # NB: as.character()ify each item in row, so we get the name in an area factor
+                    # NB: gen_param_tbl_name() will remove trailing _exp, so we don't have to worry about it from this point on
+                    sub_param_name <- gen_param_tbl_name(as.character(x[[2]]), vapply(df[i,], as.character, character(1)))
+                    sub_param_tuple <- paste0(df[i,], collapse = ".")
 
                     scope[[paste0("..param:", sub_param_name)]] <<- structure(
                         substitute(stopifnot(p %in% names(param)), list(p = sub_param_name)),
-                        param_template = df_template(sub_param_name))
+                        param_template = df_template(sub_param_name, sub_param_idx = i))
+                    init_list[[sub_param_tuple]] <- substitute(
+                        param[[name]], list(
+                            name = sub_param_name ))
                 }
+                scope[[pt_name]] <<- substitute(sym <- l, list(
+                    sym = as.symbol(pt_name),
+                    l = as.call(c(as.list(call("list")), init_list)) ))
 
-                # Replace with a  param[["lookup.cur_year.cur_step"]] call
+                # Replace with a param.lookup[["cur_year.cur_step"]] call
+                param_name_c <- as.call(c(
+                    list(as.symbol("paste")),
+                    lapply(names(df), as.symbol),
+                    list(sep = ".") ))
                 return(call('nvl',
-                    call('[[', as.symbol("param"), as.call(c(
-                        list(as.symbol("paste"), as.character(x[[2]])),
-                        lapply(names(df), as.symbol),
-                        list(sep = ".")))),
+                    call('[[', as.symbol(pt_name), param_name_c),
                     ifmissing))
             }
 
-            if (x[[1]] == 'g3_param_upper' || x[[1]] == 'g3_param_lower') {
-                # We have no bounds for R. Hard-code to NA, short-circuiting any bounds-checking
-                return(NA)
+            if (x[[1]] == 'g3_param_lower') {
+                return(substitute( param_lower[[par]], list(par = x[[2]]) ))
+            }
+            if (x[[1]] == 'g3_param_upper') {
+                return(substitute( param_upper[[par]], list(par = x[[2]]) ))
             }
 
             # Default for g3_param / g3_param_vector
@@ -111,9 +132,19 @@ g3_to_r <- function(actions, trace = FALSE, strict = FALSE) {
                     environment(fn_defn) <- env  # TODO: This should be the output function scope, not env.
                     scope[[var_name]] <<- call("<-", as.symbol(var_name), fn_defn)
                 } else if (is.character(all_defns[[var_name]]) && all_defns[[var_name]] != var_name) {
-                    # Native function with a different name
+                    # R function, not part of baseenv
                     scope[[var_name]] <<- call("<-", as.symbol(var_name), as.symbol(all_defns[[var_name]]))
                 }
+                next
+            }
+            if (is.function(all_defns[[var_name]])) {
+                # Ignore baseenv functions
+                if (identical(all_defns[[var_name]], get0(var_name, envir = baseenv()))) next
+
+                # Include function in model environment
+                # We don't modify it's environment, so closures will be intact
+                # NB: Package functions won't be visible here, since that's a "::" call.
+                assign(var_name, all_defns[[var_name]], envir = model_env)
             }
         }
 
@@ -123,7 +154,7 @@ g3_to_r <- function(actions, trace = FALSE, strict = FALSE) {
                 # Already init'ed this, ignore it.
                 next
             }
-            if (var_name == 'param') {
+            if (var_name == 'param' || var_name == 'param_lower' || var_name == 'param_upper') {
                 # It's the parameter argument
                 next
             }
@@ -184,8 +215,18 @@ g3_to_r <- function(actions, trace = FALSE, strict = FALSE) {
     }
 
     # Wrap all steps in a function call
-    out <- call("function", pairlist(param = alist(y=)$y), as.call(c(
+    out <- call("function", pairlist(param = quote( parameter_template )), as.call(c(
         list(as.symbol(open_curly_bracket)),
+        # Prefix with df -> list converstion, if needed
+        list(quote( if (is.data.frame(param)) {
+            param_lower <- structure(param$lower, names = param$switch)
+            param_upper <- structure(param$upper, names = param$switch)
+            param <- structure(param$value, names = param$switch)
+        } else {
+            # No bounds, map to NA
+            param_lower <- lapply(param, function (x) NA)
+            param_upper <- lapply(param, function (x) NA)
+        } )),
         scope,
         all_actions_code )))
 
@@ -203,16 +244,21 @@ g3_to_r <- function(actions, trace = FALSE, strict = FALSE) {
     # Turn call structure into an actual function
     out <- eval(out)
 
-    # Attach data to model as closure
+    # Generate pararameter template, add to environment for param default to use
+    model_env$parameter_template <- scope_to_parameter_template(scope, 'list')
+
+    # Attach data to model as closure, compile
     environment(out) <- model_env
+    out <- g3_r_compile(out, work_dir = work_dir, cmp_options = cmp_options)
+
     class(out) <- c("g3_r", class(out))
     attr(out, 'actions') <- actions
-    attr(out, 'parameter_template') <- scope_to_parameter_template(scope, 'list')
-    return(g3_r_compile(out))
+    attr(out, 'parameter_template') <- model_env$parameter_template
+    return(out)
 }
 
 # Generate a srcRef'ed, optimized function
-g3_r_compile <- function (model, work_dir = tempdir(), optimize = 3) {
+g3_r_compile <- function (model, work_dir = tempdir(), cmp_options = list(optimize = 3)) {
     model_string <- deparse(model)
     base_name <- paste0('g3_r_', digest::sha1(model_string))
     r_path <- paste0(file.path(work_dir, base_name), '.R')
@@ -223,13 +269,11 @@ g3_r_compile <- function (model, work_dir = tempdir(), optimize = 3) {
         NULL), r_path)
     source(r_path)
 
-    # Restore model data and attributes
+    # Restore closure from previous version
     environment(out) <- environment(model)
-    # NB: We need to do this since the environment pointers will be broken in the serialised version
-    attr(out, 'actions') <- attr(model, 'actions')
 
     # Optimize model function
-    out <- compiler::cmpfun(out, options = list(optimize = optimize))
+    if (!is.null(cmp_options)) out <- compiler::cmpfun(out, options = cmp_options)
 
     return(out)
 }
@@ -255,8 +299,11 @@ print.g3_r <- function(x, ..., with_environment = FALSE, with_template = FALSE)
     attributes(x) <- NULL
     print.function(x)
     if (with_environment) {
+        # Don't show parameter_template, we do that later
+        env_names <- setdiff(names(environment(x)), "parameter_template")
+
         writeLines("Environment:")
-        str(as.list(environment(x)), no.list = TRUE)
+        str(as.list(environment(x))[env_names], no.list = TRUE)
     }
     if (with_template) {
         writeLines("Parameter template:")
