@@ -31,11 +31,16 @@ g3s_tag <- function(inner_stock, tag_ids, force_untagged = TRUE) {
         iter_ss = c(inner_stock$iter_ss,
             tag = as.symbol("stock__tag_idx")),
         intersect = c(inner_stock$intersect, tag = quote(
-            for (stock__tag_idx in seq_along(stock__tag_ids)) {
-                if (stock__tag_ids[[stock__tag_idx]] == tag) {
-                    extension_point
-                    break
+            if (stock_isdefined(tag)) {
+                for (stock__tag_idx in seq_along(stock__tag_ids)) {
+                    if (stock__tag_ids[[stock__tag_idx]] == tag) {
+                        extension_point
+                        break
+                    }
                 }
+            } else {
+                for (stock__tag_idx in seq_along(stock__tag_ids)) g3_with(
+                    tag := stock__tag_ids[[stock__tag_idx]], extension_point)
             }
         )),
         interact = c(inner_stock$interact, tag = quote(
@@ -59,4 +64,4 @@ g3s_tag_reverse_lookup <- function (stock, tag_f) {
         keys = as.integer(stock__tag_ids),
         values = seq_along(stock__tag_ids))
     f_substitute(~g3_idx(l), list(l = lookup('getdefault', tag_f, 1L)))
-}
\ No newline at end of file
+}
