@@ -2,20 +2,23 @@ library(unittest)
 
 library(gadget3)
 
-ok(ut_cmp_equal(as.numeric(g3_eval(
-    g3_suitability_andersen(0,1,2,3,4),
-    predstock = g3_stock('pred', 11:20),
-    stock = g3_stock('prey', 1:10))), as.numeric(c(
-        "11:12" = 1.39762234830867,
-        "12:13" = 1.76710448730415,
-        "13:14" = 1.92001105688051,
-        "14:15" = 1.98080983811489,
-        "15:16" = 1.99913177015507,
-        "16:17" = 1.99765923114992,
-        "17:18" = 1.98837494524607,
-        "18:19" = 1.97544436079439,
-        "19:20" = 1.96094041453242,
-        "20:Inf" = 1.94597890503981))), "g3_eval: substituted 2 stocks")
+ok(ut_cmp_equal(
+    g3_eval(
+        g3_suitability_andersen(0,1,2,3,4),
+        predator_length = 99,
+        stock = g3_stock('prey', 1:10 * 10) ),
+    gadget3:::force_vector(c(
+        "10:20" = 1.5385642245386,
+        "20:30" = 1.90781891144376,
+        "30:40" = 1.99894574784489,
+        "40:50" = 1.97774955212781,
+        "50:60" = 1.91681934676797,
+        "60:70" = 1.83906905397844,
+        "70:80" = 1.75539377253919,
+        "80:90" = 1.67124659054669,
+        "90:100" = 1.58937906439558,
+        "100:Inf" = 1.51113553695681,
+        NULL ))), "g3_eval: substituted stock & iterator value")
 
 ok(ut_cmp_equal(
     g3_eval(quote( g3_param('x', value = 99) )),
